#!/usr/bin/python
# -*- coding: utf-8 -*-
###! C:\python27\python.exe
####################################################################################
#	Ellipsoid
#
# Ver = '1.0'; Date = '2017-1-31'	# Initial version (no CGI)
# Ver = '1.1'; Date = '2017-2-5'	# Initial CGI version
# Ver = '1.11'; Date = '2017-2-8'	# Parameters added (evfac, etc.)
# Ver = '1.12'; Date = '2017-2-9'
# Ver = '1.13'; Date = '2017-2-12'	# Support updated
# Ver = '1.14'; Date = '2017-2-14'	# Support updated
# Ver = '1.15'; Date = '2017-2-20'	# Summary string updated (int to int(...+0.5)), wave changed
# Ver = '1.16'; Date = '2017-2-20'	# Vibrato introduced
# Ver = '1.17'; Date = '2017-2-26'	# Form field values checked (float, int)
# Ver = '1.18'; Date = '2017-2-28'	# getvalue() introduced (it returns '' if form field is '')
Ver = '1.19'; Date = '2017-3-3'		# Rounding (int(...+0.5)) debugged
#
#	Copyright (C) 2017 by Yasusi Kanada, Dasyn.com
####################################################################################

import draw3dp
from math import sin, cos
import cgi, sys


VPower = 1.25
CPower = 1.25


## Mathematical constants ##

PI = 3.14159265359


## Basic (object) parameters ##

# Default shape and size of ellipsoid #
C0 = 20
AB0 = 20
# rfac = AB / float(C)

# Top-hole control -- 0 <= Height <= 2 * C. no hole if Height == 2 * C
Height0 = 300 # 2 * C0  # initial height

# Bottom-hole control -- 0 <= HeightZ < 2 * C. no hole if HeightZ <= 0
HeightZ0 = 0

# Additional parameters #
wcyc0 = 3
wfac0 = 0.0
efac0 = 0.9
evfac0 = 1.0


## Pitch parameters ##

pitch00 = 0.30 # 0.35


## Printer parameter ##

IsDelta = True

IsABS = False  # PLA

# Printer = 'Kossel1'
Printer = 'Kossel2'
# Printer = 'Kossel3'


## Printbed parameters ##

if IsDelta:
    x0 = 0
    y0 = 0
else:
    x0 = 100
    y0 = 100


## Extrusion parameters ##

defaultCrossSection = 0.20 # Kossel
# defaultCrossSection = 0.42 # Rostock

thickCrossSection = 0.20 # Kossel

if IsDelta:
    FilamentDiameter = 1.75
else:
    FilamentDiameter = 2.90


## Default velocity ##

DefaultVelocity = 30.0


## Temperature parameters ##

HeadTemperature  = 250
HeadTemperature1 = 240
HeadTemperature2 = 220 # 230

if IsABS:
    BedTemperature  = 90
else: # PLA
    BedTemperature  = 32


## Modulation/deformation function ##

def fmd(theta, z, pmd, C):
    result = 1.0
    for i in range(len(pmd)):
        type = pmd[i]['type']
        a  = pmd[i]['amplitude']
        fr = pmd[i]['freqR']
        fz = pmd[i]['freqZ']
        theta0 = pmd[i]['theta0']
        wave = sin(fr*theta + PI*fz*z/C + theta0)
        if type == 'sin':
            wave = a * wave
        elif type == 'clip':
            wave = a if wave >= a else -a if wave <= -a else wave
        elif type == 'rectangular':
            wave = a if wave >= 0 else -a
        else:
            wave = 0
        result += wave
    return result


## Vibrato function ##

def fv(theta, z, pmd, Height, C):
    result = 0
    for i in range(len(pmd)):
        a  = pmd[i]['amplitude']
        fr = pmd[i]['freqR']
        fz = pmd[i]['freqZ']
        theta0 = pmd[i]['theta0']
        result += a * cos(fr*theta + PI*fz*z/C + theta0)
    return z + (Height - z) * z * result


## Skirt and support ##

def draw_skirt(obj, size = 20, z = 0.4):
    obj.comment("skirt")
    obj.lineto(-size,  size, z)
    obj.lineto( size,  size, z)
    obj.lineto( size, -size, z)
    obj.lineto(-size, -size, z)
    obj.lineto( size,  size, z)
    obj.lineto(  0,  size, z)
    obj.lineto(  0, -size, z)
    obj.lineto( size, -size, z)
    obj.lineto(-size,  size, z)
    obj.lineto(-size,   0, z)
    obj.lineto( size,   0, z)
    obj.moveto(0, 0, z)
    return

def support(obj, size = 15, r = 1.0, r2 = 0.4, h = 1.0, vpitch = 0.4, hpitch = 0.5,
            skirt = 'double', gap = 0.8):
    obj.comment("support")
    draw_skirt(obj, size = size, z = vpitch)
    if skirt == 'double':
        draw_skirt(obj, size = size, z = 2 * vpitch)
        height = 2 * vpitch + h
    else:
        height = vpitch + h
    if r > 0:
        obj.helix(r, h, vpitch, 0, 0, height - h + gap)
        obj.cylinder(r2, h, vpitch, hpitch, 0, 0, height - h)
        # obj.circle(r, 0, 0, height + gap - h)
    return height + gap


## Input parameters ##

def getvalue(form, attr):
    if attr in form:
        return form[attr].value
    else:
        return ''


def get_parameters(form, editFunction):
    pd = []
    pm = []
    pv = []
    if 'phase' in form: # non-initial phase
        ## Basic parameters ##
        AB = form.getvalue('AB')
        C = form.getvalue('C')
        pitch0 = getvalue(form, 'pitch0')
        Height = getvalue(form, 'Height')
        HeightZ = getvalue(form, 'HeightZ')

        # Additional wave parameters #
        efac = getvalue(form, 'efac')
        evfac = getvalue(form, 'evfac')

        ## Deformation parameters ##
        n_pd = getvalue(form, 'n_pd')
        if n_pd != '':
            for i in range(int(n_pd)):
                si = str(i)
                pd.append({'type': getvalue(form, 'dtype_' + si),
                           'amplitude': getvalue(form, 'damplitude_' + si),
                           'freqR': getvalue(form, 'dfreqR_' + si),
                           'freqZ': getvalue(form, 'dfreqZ_' + si),
                           'theta0': getvalue(form, 'dtheta0_' + si)})

            # Process edit operations #
            if editFunction['type'] == 'dinsert':
                pd = sum([pd[:editFunction['location']+1],
                          [{'type': 'sin', 'amplitude': '0',
                            'freqR': '0', 'freqZ': '0', 'theta0': '0'}],
                          pd[editFunction['location']+1:]],
                         [])
            elif editFunction['type'] == 'ddelete':
                pd = sum([pd[:editFunction['location']],
                          pd[editFunction['location']+1:]],
                         [])

        ## Modulation parameters ##
        n_pm = getvalue(form, 'n_pm')
        if n_pm != '':
            for i in range(int(n_pm)):
                si = str(i)
                pm.append({'type': getvalue(form, 'mtype_' + si),
                           'amplitude': getvalue(form, 'mamplitude_' + si),
                           'freqR': getvalue(form, 'mfreqR_' + si),
                           'freqZ': getvalue(form, 'mfreqZ_' + si),
                           'theta0': getvalue(form, 'mtheta0_' + si)})

            # Process edit operations #
            if editFunction['type'] == 'minsert':
                pm = sum([pm[:editFunction['location']+1],
                          [{'type': 'sin', 'amplitude': '0',
                            'freqR': '0', 'freqZ': '0', 'theta0': '0'}],
                          pm[editFunction['location']+1:]],
                         [])
            elif editFunction['type'] == 'mdelete':
                pm = sum([pm[:editFunction['location']],
                          pm[editFunction['location']+1:]],
                         [])

        ## Vibrato parameters ##
        n_pv = getvalue(form, 'n_pv')
        if n_pv != '':
            for i in range(int(n_pv)):
                si = str(i)
                pv.append({'type': getvalue(form, 'vtype_' + si),
                           'amplitude': getvalue(form, 'vamplitude_' + si),
                           'freqR': getvalue(form, 'vfreqR_' + si),
                           'freqZ': getvalue(form, 'vfreqZ_' + si),
                           'theta0': getvalue(form, 'vtheta0_' + si)})

            # Process edit operations #
            if editFunction['type'] == 'vinsert':
                pv = sum([pv[:editFunction['location']+1],
                          [{'type': 'sin', 'amplitude': '0',
                            'freqR': '0', 'freqZ': '0', 'theta0': '0'}],
                          pv[editFunction['location']+1:]],
                         [])
            elif editFunction['type'] == 'vdelete':
                pv = sum([pv[:editFunction['location']],
                          pv[editFunction['location']+1:]],
                         [])

    else:
        ## Basic parameters ##
        AB = AB0
        C = C0
        pitch0 = pitch00
        Height = Height0
        HeightZ = HeightZ0

        # Additional parameters #
        efac = efac0
        evfac = evfac0

        ## Deformation parameters ##
        pd = [{'type':'sin', 'amplitude':'0',
               'freqR':'0', 'freqZ':'0', 'theta0':'0'}]

        ## Modulation parameters ##
        pm = [{'type':'sin', 'amplitude':'0',
               'freqR':'0', 'freqZ':'0', 'theta0':'0'}]

        ## Vibrato parameters ##
        pv = [{'type':'sin', 'amplitude':str(wfac0),
               'freqR':str(wcyc0), 'freqZ':'0', 'theta0':'0'}]

    print('''<form action="ellipsoid.cgi" method="get">
          <input type="hidden" name="phase" value="parameters" />
          <input type="hidden" name="n_pm" value="''' + str(len(pm)) + '''" />
          <input type="hidden" name="n_pd" value="''' + str(len(pd)) + '''" />
          <input type="hidden" name="n_pv" value="''' + str(len(pv)) + '''" />

          <h2>Basic parameters</h2>

          <label for="AB">ab = a = b =</label>
          <input type="text" name="AB" value="''' + str(AB) + '''" />
          (10 &le; ab &le; 60)<br />

          <label for "C">c =</label>
          <input type="text" name="C" value="''' + str(C) + '''" />
          (20 &le; c &le; 60)<br />

          <label for "pitch0">filament pitch =</label>
          <input type="text" name="pitch0" value="''' + str(pitch0) + '''" />
          (0.2 &le; pitch &le; 0.4)<br />

          <label for "Height">height =</label>
          <input type="text" name="Height" value="''' + str(Height) + '''" />
          (1 &le; height/c &le; 2 &mdash; "2" means no top hole)<br />

          <label for "height0">height0 =</label>
          <input type="text" name="HeightZ" value="''' + str(HeightZ) + '''" />
          (0 &le; height0/c &lt; height/c &mdash; "0" means no bottom hole)

          <p>
          <img src="/ellipsoid_model.png" />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
          <img src="/shadeShape.png" />
          </p>

          <h2>Modulations</h2>

          <table>
            <tr>
              <td>Wave type</td>
              <td>Amplitude (0 .. 0.3)</td>
              <td>Cycle(a, b) (-40 .. 40)</td>
              <td>Cycle(c) (-20 .. 20)</td>
              <td>Shift (radian)</td>
              <td>Operations</td>
            </tr>''')

    for i in range(len(pm)):
        si = str(i)
        type = pm[i]['type']
        print('''
            <tr>
              <td>
                <select name ="mtype_''' + si + '''">
                  <option value="sin"''' + (' selected' if type == 'sin' else '') + '''>sin</option>
                  <option value="clip"''' + (' selected' if type == 'clip' else '') + '''>clip</option>
                  <!-- option value="rectangular"''' + (' selected' if type == 'rectangular' else '') + '''>rectangular</option -->
                </select>
              </td>
              <td>
                <input type="text" name="mamplitude_''' + si + '''" value="''' + pm[i]['amplitude'] + '''" />
              </td>
              <td>
                <input type="text" name="mfreqR_''' + si + '''" value="''' + pm[i]['freqR'] + '''" />
              </td>
              <td>
                <input type="text" name="mfreqZ_''' + si + '''" value="''' + pm[i]['freqZ'] + '''" />
              </td>
              <td>
                <input type="text" name="mtheta0_''' + si + '''" value="''' + pm[i]['theta0'] + '''" />
              </td>
              <td>
                <input type="submit" value="insert" name="minsert''' + si + '''" />''')
        print('''
                <input type="submit" value="delete" name="mdelete''' + si + '''" />''' if len(pm) > 1 else '')
        print('''
              </td>
            </tr>''')
    print('''
          </table>

          <h2>Deformations</h2>

          <table>
            <tr>
              <td>Wave type</td>
              <td>Amplitude (0 .. 0.1)</td>
              <td>Cycle(a, b) (-40 .. 40)</td>
              <td>Cycle(c) (-20 .. 20)</td>
              <td>Shift (radian)</td>
              <td>Operations</td>
            </tr>''')

    for i in range(len(pd)):
        si = str(i)
        type = pd[i]['type']
        print('''
            <tr>
              <td>
                <select name ="dtype_''' + si + '''">
                  <option value="sin"''' + (' selected' if type == 'sin' else '') + '''>sin</option>
                  <option value="clip"''' + (' selected' if type == 'clip' else '') + '''>clip</option>
                  <!-- option value="rectangular"''' + (' selected' if type == 'rectangular' else '') + '''>rectangular</option -->
                </select>
              </td>
              <td>
                <input type="text" name="damplitude_''' + si + '''" value="''' + pd[i]['amplitude'] + '''" />
              </td>
              <td>
                <input type="text" name="dfreqR_''' + si + '''" value="''' + pd[i]['freqR'] + '''" />
              </td>
              <td>
                <input type="text" name="dfreqZ_''' + si + '''" value="''' + pd[i]['freqZ'] + '''" />
              </td>
              <td>
                <input type="text" name="dtheta0_''' + si + '''" value="''' + pd[i]['theta0'] + '''" />
              </td>
              <td>
                <input type="submit" value="insert" name="dinsert''' + si + '''" />''')
        print('''
                <input type="submit" value="delete" name="ddelete''' + si + '''" />''' if len(pd) > 1 else '')
        print('''
              </td>
            </tr>''')
    print('''
          </table>

          <h2>Vibrato</h2>

          <table>
            <tr>
              <td>Wave type</td>
              <td>Amplitude (0 .. 0.002)</td>
              <td>Cycle(a, b) (-40 .. 40)</td>
              <td>Cycle(c) (-20 .. 20)</td>
              <td>Shift (radian)</td>
              <td>Operations</td>
            </tr>''')

    for i in range(len(pv)):
        si = str(i)
        type = pv[i]['type']
        print('''
            <tr>
              <td>
                <select name ="vtype_''' + si + '''">
                  <option value="sin"''' + (' selected' if type == 'sin' else '') + '''>sin</option>
                </select>
              </td>
              <td>
                <input type="text" name="vamplitude_''' + si + '''" value="''' + pv[i]['amplitude'] + '''" />
              </td>
              <td>
                <input type="text" name="vfreqR_''' + si + '''" value="''' + pv[i]['freqR'] + '''" />
              </td>
              <td>
                <input type="text" name="vfreqZ_''' + si + '''" value="''' + pv[i]['freqZ'] + '''" />
              </td>
              <td>
                <input type="text" name="vtheta0_''' + si + '''" value="''' + pv[i]['theta0'] + '''" />
              </td>
              <td>
                <input type="submit" value="insert" name="vinsert''' + si + '''" />''')
        print('''
                <input type="submit" value="delete" name="vdelete''' + si + '''" />''' if len(pv) > 1 else '')
        print('''
              </td>
            </tr>''')
    print('''
          </table>

          <h2>Additional parameters</h2>

          <label for "efac">extrusion factor (efac) =</label>
          <input type="text" name="efac" value="''' + str(efac) + '''" />
          (0.5 &lt; efac &lt; 2.0)<br />

          <label for "evfac">head motion speed factor (evfac) =</label>
          <input type="text" name="evfac" value="''' + str(evfac) + '''" />
          (0.7 &lt; evfac &lt; 1.4)

          <br /><br />
          <input type="submit" value="Generate code" name="codegen" />
</form>
''')


## G-code generation ##

def getint(form, field, printName):
    if form.getvalue(field) == None:
        print('*** Value error in {}: {}'.format(printName, 'No value ("")'))
        return 0
    else:
        try:
            return int(form[field].value)
        except ValueError:
            print('*** Value error in {}: "{}" (must be integer)'.format(printName, form[field].value))
            return 0


def getfloat(form, field, printName):
    if form.getvalue(field) == None:
        print('*** Value error in {}: {}'.format(printName, 'No value ("")'))
        return 0.0
    else:
        try:
            return float(form[field].value)
        except ValueError:
            print('*** Value error in {}: "{}"'.format(printName, form[field].value))
            return 0.0


def gen_gcode(form):
    print(';### G-Code ###')
    print(';')

    ## Get parameters ##

    # Basic object parameters #
    AB = getfloat(form, 'AB', 'ab')
    C = getfloat(form, 'C', 'c')
    if C == 0:
        C == AB  # avoid zero division
    rfac = AB / C
    pitch0 = getfloat(form, 'pitch0', 'pitch')
    Height = getfloat(form, 'Height', 'height')
    HeightZ = getfloat(form, 'HeightZ', 'height0')
    if Height > 2 * C:
        Height = 2 * C

    # Additional parameters #
    efac = getfloat(form, 'efac', 'efac')
    evfac = getfloat(form, 'evfac', 'evfac')

    # Deformation parameters #
    pd = []
    n_pd = form['n_pd'].value
    for i in range(int(n_pd)):
        si = str(i)
        pd.append({'type': form['dtype_' + si].value,
                   'amplitude': getfloat(form, 'damplitude_' + si, 'deformation amplitude'),
                   'freqR': getint(form, 'dfreqR_' + si, 'deformation cycle(a, b)'),
                   'freqZ': getfloat(form, 'dfreqZ_' + si, 'deformation cycle(c)'),
                   'theta0': getfloat(form, 'dtheta0_' + si, 'deformation shift')})

    # Modulation parameters #
    pm = []
    n_pm = form['n_pm'].value
    for i in range(int(n_pm)):
        si = str(i)
        pm.append({'type': form['mtype_' + si].value,
                   'amplitude': getfloat(form, 'mamplitude_' + si, 'modulation amplitude'),
                   'freqR': getint(form, 'mfreqR_' + si, 'modulation cycle(a, b)'),
                   'freqZ': getfloat(form, 'mfreqZ_' + si, 'modulation cycle(c)'),
                   'theta0': getfloat(form, 'mtheta0_' + si, 'modulation shift')})

    # Vibraro parameters #
    pv = []
    n_pv = form['n_pv'].value
    for i in range(int(n_pv)):
        si = str(i)
        pv.append({'type': form['vtype_' + si].value,
                   'amplitude': getfloat(form, 'vamplitude_' + si, 'vibrato amplitude'),
                   'freqR': getint(form, 'vfreqR_' + si, 'vibrato cycle(a, b)'),
                   'freqZ': getfloat(form, 'vfreqZ_' + si, 'vibrato cycle(c)'),
                   'theta0': getfloat(form, 'vtheta0_' + si, 'vibrato shift')})

    ## Print parameter summary ##
    summary = 'AB{}C{}P{}H{}-{}E{}-{}'.\
        format(int(round(AB)), int(round(C)), int(round(100*pitch0)),
	       int(round(HeightZ)), int(round(Height)),
               int(round(100*efac)), int(round(100*evfac)))
    summary += 'M'
    for i in range(len(pm)):
        if pm[i]['amplitude'] != 0:
            summary += '$' + pm[i]['type'][0] + 'A' + str(int(round(100*pm[i]['amplitude']))) + \
                'F' + str(int(round(pm[i]['freqR']))) + '-' + str(int(round(pm[i]['freqZ']))) + \
                'T' + str(int(round(100*pm[i]['theta0'])))
    summary += 'D'
    for i in range(len(pd)):
        if pd[i]['amplitude'] != 0:
                summary += '$' + pd[i]['type'][0] + 'A' + str(int(round(100*pd[i]['amplitude']))) + \
                'F' + str(int(round(pd[i]['freqR']))) + '-' + str(int(round(pd[i]['freqZ']))) + \
                'T' + str(int(round(100*pd[i]['theta0'])))
    summary += 'V'
    for i in range(len(pv)):
        if pv[i]['amplitude'] != 0:
                summary += '$' + pv[i]['type'][0] + 'A' + str(int(round(10000*pv[i]['amplitude']))) + \
                'F' + str(int(round(pv[i]['freqR']))) + '-' + str(int(round(pv[i]['freqZ']))) + \
                'T' + str(int(round(100*pv[i]['theta0'])))
    print(';*** Parameter summary: ' + summary + '.gcode')
    print(';')


    ## Initialize draw3dp ##
    draw3dp.init(FilamentDiameter, 250, BedTemperature, DefaultVelocity)


    ## Generate support ##
    sup = draw3dp.Trace(thickCrossSection, 0, 0, 0.4)
    supportHeight = support(sup, size = 20, r = 3.0, h = 1.2, vpitch = 0.4,
        		    skirt = 'single', gap = 0.5)
    sup.draw(supportHeight)

    print(';')
    print('; AB={} C={} pitch0={} Height={} HeightZ={}'.format(AB, C, pitch0, Height, HeightZ))


    ## Create initial shade form (helix) ##
    obj = draw3dp.Trace(defaultCrossSection, x0, y0, 0.0)

    pitch = 2 * pitch0 * C / (AB + C)
    vfacout = 0.9; vfacin = 0.12; cfacout = 0.60; cfacin = 0.50
    # vfacout = 0.9; vfacin = 0.12; cfacout = 0.50; cfacin = 0.30
    # vfacout = 0.9; vfacin = 0.15; cfacout = 0.50; cfacin = 0.25

    obj.helix(C, Height, pitch, 0, 0, 0, minlen=0.2)

    if Printer == 'Kossel1' or Printer == 'Kossel3':
        cfacout *= 1.06; cfacin *= 1.06
    if Printer == 'Kossel3':
        vfacout *= 0.8


    ## Vibrato ##
    print('; Vibrato pv={}'.format(pv))
    obj.deform_cylinder(
        lambda r, theta, z: (r, theta, fv(theta, z, pv, Height, C)),
        lambda v, r, theta, z: v,
        lambda c, r, theta, z: c)

    # Temporary vertical translation #
    obj.deform_cylinder(
        lambda r, theta, z: (r, theta, z - C),
        lambda v, r, theta, z: v,
        lambda c, r, theta, z: c)


    ## Adjust filament density (for ellipsoid) ##
    if rfac <= 0.67:
        z1 = 4 * C * (1 - rfac)
    elif rfac >= 1.5:
        z1 = -2 * C * rfac
    else:
        z1 = 1e99
    print('; z1={}'.format(z1))

    obj.deform_cylinder(
        lambda r, theta, z:
            (r, theta, (z+z1)/(C+z1) * z if z > 0 else (z1-z)/(C+z1) * z),
        lambda v, r, theta, z: v,
        lambda c, r, theta, z: c)


    ## Deform to sphere ##
    z0 = C * sin(0.5*PI*C/C) - HeightZ + supportHeight

    obj.deform_cylinder(
        lambda r, theta, z:
            (rfac * r * cos(0.5*PI*z/C), theta, r * sin(0.5*PI*z/C)),
            # (rfac * r * cos(0.5*PI*z/C), theta, r * sin(0.5*PI*z/C) + z0),
        lambda v, r, theta, z: v,
        lambda c, r, theta, z: c)


    ## Modulation ##
    print('; Modulation pm={}'.format(pm))
    obj.modulate_cylinder(lambda theta, z: fmd(theta, z, pm, C))


    ## Deformation ##
    print('; Deformation pd={}'.format(pd))
    obj.deform_cylinder(
        lambda r, theta, z: (r * fmd(theta, z, pd, C), theta, z),
        # lambda r, theta, z: (r * fmd(theta, z, pd, C), theta, z + z0),
        lambda v, r, theta, z: v,
        lambda c, r, theta, z: c)


    ## Printing speed and extrusion speed control ##
    obj.deform_cylinder_t(
        lambda r, theta, z: (r, theta, z + z0),
        lambda v, r, theta, z:
            evfac * (vfacout * (r/AB)**VPower + vfacin * (1 - (r/AB)**VPower)) * v * 1.3,
        lambda c, r, theta, z:
            efac * (cfacout * (r/AB)**CPower + cfacin * (1 - (r/AB)**CPower)) * c,
        lambda t, r, theta, z: HeadTemperature2 if z < -0.93 * C or 0.7 * C < z else \
            HeadTemperature1 if z < -0.86 * C or 0.2 * C < z else HeadTemperature)

    ## Drawing ##
    obj.draw(minMotion = 0.4)
    print('; END')


def gen_html_head():
    print('''Content-Type: text/html

<html>
<head><title>3D-printed shade designer</title></head>
<body>

<h1>3D-printed shade designer</h1>

<p>''' + Date + ', ver. ' + Ver + '''</p>
''')


def gen_text_head():
    print('''Content-Type: text/plain
''')


def gen_html_tail():
    print('''</body>
</html>
''')


## Main ##

form = cgi.FieldStorage()

if 'codegen' in form:
    gen_text_head()

    gen_gcode(form)
else: # Non-final step
    gen_html_head()

    editFunction = {}
    if 'n_pm' in form:
        n_pm = int(form['n_pm'].value)
        for i in range(n_pm):
            mi = 'minsert' + str(i)
            if mi in form:
                editFunction = {'type': 'minsert', 'location': i}
            md = 'mdelete' + str(i)
            if md in form:
                editFunction = {'type': 'mdelete', 'location': i}
    if 'n_pd' in form:
        n_pd = int(form['n_pd'].value)
        for i in range(n_pd):
            di = 'dinsert' + str(i)
            if di in form:
                editFunction = {'type': 'dinsert', 'location': i}
            dd = 'ddelete' + str(i)
            if dd in form:
                editFunction = {'type': 'ddelete', 'location': i}
    if 'n_pv' in form:
        n_pv = int(form['n_pv'].value)
        for i in range(n_pv):
            vi = 'vinsert' + str(i)
            if vi in form:
                editFunction = {'type': 'vinsert', 'location': i}
            vd = 'vdelete' + str(i)
            if vd in form:
                editFunction = {'type': 'vdelete', 'location': i}

    get_parameters(form, editFunction)

    gen_html_tail()
